<?php

namespace FiloBlu\Storelocator\Controller\PickUpInStore;

use FiloBlu\Storelocator\Helper\Data;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Registry;
use Magento\Framework\View\Result\PageFactory;
use Magento\Sales\Api\Data\OrderInterfaceFactory;

/**
 *
 */
class Confirm extends Action
{
    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var \FiloBlu\Storelocator\Helper\Data
     */
    protected $helper;

    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterfaceFactory
     */
    private $orderFactory;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \FiloBlu\Storelocator\Helper\Data $helper
     * @param \Magento\Sales\Api\Data\OrderInterfaceFactory $orderFactory
     * @param \Magento\Framework\Registry $registry
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        Data $helper,
        OrderInterfaceFactory $orderFactory,
        Registry $registry
    ) {
        parent::__construct($context);
        $this->resultPageFactory = $resultPageFactory;
        $this->helper = $helper;
        $this->orderFactory = $orderFactory;
        $this->registry = $registry;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface|\Magento\Framework\View\Result\Page|void
     */
    public function execute()
    {
        $incrementId = $this->getRequest()->getParam('increment_id');
        $hash = $this->getRequest()->getParam('hash');

        if (!$this->helper->getConfig('carriers/pickupinstore/active') || !$incrementId || !$hash) {
            $this->_forward('noRoute');
            return;
        }

        $order = $this->orderFactory->create();
        $order->loadByIncrementId($incrementId);

        if ($order->getId() && $order->getPickupInStoreHash() == $hash && $order->getShippingMethod(
            ) == 'pickupinstore_pickupinstore') {
            $this->registry->register('pickupinstore_order_confirm', $order);
            $resultPage = $this->resultPageFactory->create();
            $resultPage->getConfig()->getTitle()->set(
                __('PickUp In Store - Confirm customer pick up for order ') . $incrementId
            );

            return $resultPage;
        } else {
            $this->_forward('noRoute');
        }
    }
}
